# -*- coding: utf-8 -*-

# AutoCloseHelper.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import re

class AutoCloseHelper:
	reEnvironment = None

	def __init__(self, documentWrapper):
		self.documentWrapper = documentWrapper
		self.textBuffer = documentWrapper.textBuffer
	
	def autoClose(self):
		insIter = self.textBuffer.get_iter_at_mark(self.textBuffer.get_insert())
		limitIter = insIter.copy()
		limitIter.backward_chars(30)
		match = insIter.backward_search("\\", False, limitIter)
			
		if not match:
			return False
			
		if not self.reEnvironment:
			self.reEnvironment = re.compile(r"begin\{([a-zA-Z\*]+)\}$")
			
		string = self.textBuffer.get_text(insIter, match[1], False)
		match = self.reEnvironment.match(string)
		if match:
			if not self.documentWrapper.isLatex():
				return False
				
			indent = self.documentWrapper.getIndentString(insIter.get_line())
			closeString = "\n%s\t\n%s\\end{%s}" % (indent, indent, match.group(1))
				
			mark = self.textBuffer.create_mark(None, self.textBuffer.get_iter_at_mark(self.textBuffer.get_insert()), True)
				
			self.textBuffer.insert_at_cursor(closeString)
				
			insIter = self.textBuffer.get_iter_at_mark(mark)
			insIter.forward_line()
			insIter.forward_to_line_end()
			self.textBuffer.place_cursor(insIter)
			self.textBuffer.delete_mark(mark)
			
